from __future__ import unicode_literals

import os
import shutil

import pytest


@pytest.fixture(scope="session")
def image(request):
    """
    parse the argument in the test else let the tests skip
    """
    image_value = "tests/test_files/plain-white-background.jpg"
    if image_value is None:
        pytest.skip()
    return image_value  # the image file name


@pytest.fixture(scope="session")
def video(request):
    """
    parse video to test else skip
    :param request:
    :return:
    """
    video_value = "tests/test_files/earth.mp4"

    if video_value is None:
        pytest.skip()
    return video_value


@pytest.fixture
def datadir(tmpdir, request):
    """
    Fixture responsible for searching a folder with the same name of test
    module and, if available, moving all contents to a temporary directory so
    tests can use them freely.
    """
    filename = request.module.__file__
    test_dir, _ = os.path.splitext(filename)

    if os.path.isdir(test_dir):
        shutil.copytree(test_dir, str(tmpdir), dirs_exist_ok=True)

    return tmpdir



"""
.. _module_colShare
Module ColShare
===============
Global methods non-object specific

Usage:


>>> from colordetect import col_share
# show a progress bar for a process
>>> col_share.progress_bar("<current_process_position>", "<total_process_length>", "<process_description>")
# sort a dictionary by value to required length or in specific order
>>> col_share.sort_order('<dictionary>', "<items_to_return>", "<order>")
"""

import logging
import sys
from urllib.parse import urlparse

LOGGER = logging.getLogger(__name__)


def progress_bar(position: int, total_length: int, post_text: str = "Color Detection"):
    """
    progress_bar
    ----------------
    Display a progress bar of video processing

    Parameters
    ----------
    position: int
        Current position of process
    total_length: int
        Total length of process
    post_text: str
        Text to display along with progress bar
    """
    n_bar = 100
    #       # size of progress bar
    j = position / total_length
    sys.stdout.write("\r")
    sys.stdout.write(f"[{'#' * int(n_bar * j):{n_bar}s}] {int(100 * j)}% {post_text}")
    sys.stdout.flush()


def sort_order(object_description: dict, key_count: int = 5, ascending: bool = True):
    """
    .. _color_sort
    sort_order
    ----------------
    Sort items in a dictionary according to value

    Parameters
    ----------
    object_description: dict
        A dictionary whose values need sorting
    key_count: int
        The number of items to return from the sort
    ascending: bool
        The order to perform the dictionary sort. By default, set to True.
    :return: A sorted dictionary with specific number
    """
    if not isinstance(key_count, int):
        raise TypeError(
            f"color_count has to be an integer. Provided {type(key_count)} "
        )

    if not isinstance(ascending, bool):
        raise TypeError(
            f"The value of the 'ascending' parameter is a boolean. Provided {type(ascending)} "
        )

    sorted_colors = {
        k: v
        for k, v in sorted(
            object_description.items(), key=lambda item: item[1], reverse=ascending
        )
    }
    return dict(list(sorted_colors.items())[0:key_count])


def is_url(url: str) -> bool:
    """
    Check if the string parsed is a URL

    Parameters
    ----------

    url: str
        A string to be checked
    """
    try:
        result = urlparse(url)
        return all([result, result.scheme, result.netloc, result.path])
    except Exception as e:
        LOGGER.info(f"String passed is not an image.{e}")
        return False



"""
test whether file passed is an image/video
test whether text is written to the image
"""

import mimetypes
from pathlib import Path

import matplotlib.colors as mcolors

from ..colordetect import ColorDetect


def test_existence_of_image_vid_path(image, video):
    """
    Test whether the filename used in the test is the first image
    """
    assert Path(image).exists()
    assert Path(video).exists()


def test_argument_is_image(image):
    """
    Ensure that the argument parsed is an image
    """
    mime_type, _ = mimetypes.guess_type(str(image))
    assert mime_type is not None and mime_type.startswith("image/")


def test_video_passed_is_valid_video(video):
    """

    :param video:
    :return:
    """
    assert mimetypes.guess_type(video)[0].rsplit("/")[0] == "video"


def test_image_has_been_saved(datadir, image):
    """
    Ensure that 'out.jpg' has been saved to the choice directory
    """
    user_image = ColorDetect(image)
    user_image.get_color_count(color_count=1)
    file_name = "out.jpg"
    user_image.save_image(location=datadir, file_name=file_name)
    assert Path(datadir / file_name).exists()


def test_correct_color_format(image):
    """
    What user specified is the color format to be written is being written
    :param image:
    :return:
    """
    user_image = ColorDetect(image)
    #  test rgb returns rgb color format
    color_codes = user_image.get_color_count(color_count=1, color_format="hex")
    color_code = list(color_codes.keys())[0]
    assert mcolors.is_color_like(color_code)


def test_text_in_image(datadir):
    """
    output image should have text
    """
    # Todo
    pass



"""
Unit tests: testing a small bit of code like function or class in isolation of the system
From the developers perspective
"""

import os
from pathlib import Path

import pytest

from ..colordetect import ColorDetect, VideoColor, col_share


def test_image_vid_parsed_to_class(image, video):
    """
    test whether an image/video is parsed to the class ColorDetect(<image>) and VideoColor(<video>)
    Check whether an instance is created
    """
    isinstance(ColorDetect(image), ColorDetect)
    isinstance(VideoColor(video), VideoColor)


def test_color_detect_gets_numpy_array_from_video(image, video):
    """
    Test whether the filename used in the test is the first image
    """
    # user_video = VideoColor(video)
    pass


def test_get_color_count_has_correct_color_and_count(image):
    """
    Ensure get_color_count gets the correct color and count
    """
    user_image = ColorDetect(image)
    # since the image is plain 255,255,255
    assert len(user_image.get_color_count(color_count=1)) == 1
    assert user_image.get_color_count(color_count=1) == {"white": 100.0}


def test_image_saving(datadir, image):
    """
    Ensure the image is saved and arguments parsed are valid
    """

    user_image = ColorDetect(image)

    user_image.get_color_count(color_count=1)
    file_name = "out.jpg"

    with pytest.raises(NotADirectoryError):
        user_image.save_image(location="/non_existent_folder", file_name=file_name)

    user_image.save_image(location=datadir, file_name=file_name)
    saved_image = Path(os.path.join(datadir, file_name))

    assert Path.exists(saved_image)


def test_valid_color_format_is_parsed(image, video):
    """
    An exception is raised if an invalid color_format is parsed
    """
    user_image = ColorDetect(image)
    user_video = VideoColor(video)

    with pytest.raises(Exception):
        user_image.get_color_count(color_count=1, color_format="invalid_random_format")

    with pytest.raises(Exception):
        user_video.get_video_frames(
            frame_color_count=1, color_format="invalid_random_format"
        )

    user_image.get_color_count(color_count=1)
    with pytest.raises(Exception):
        user_image.write_color_count(font_color=(267, 0, 0))


def test_valid_params_to_get_color_count(image):
    """
    An exception is raised if an invalid color_count value is parsed. Instance, a string
    """
    user_image = ColorDetect(image)
    with pytest.raises(Exception):
        user_image.get_color_count(color_count="many_colors")


def test_save_params_are_valid(image, datadir):
    """
    A string is being used as a file name as well as location
    """
    user_image = ColorDetect(image)
    user_image.get_color_count(color_count=1)

    with pytest.raises(Exception):
        user_image.save_image(location=datadir, file_name=5)

    # with pytest.raises(Exception) as e_info:
    #     user_image.save_image(location=500, file_name="output.jpg")


def test_result_file_name_is_valid(image, datadir):
    """
    test result filename has what was given as the file name
    :param image:
    :param datadir:
    :return:
    """
    user_image = ColorDetect(image)
    user_image.get_color_count(color_count=1)
    file_name = "ramble.jpg"
    user_image.save_image(location=datadir, file_name=file_name)
    saved_file = os.path.basename(Path(datadir / file_name))
    assert saved_file == file_name


def test_progress_bar_shows_correct_percentage(video):
    """
    ensure the percentage displayed is correct
    :param video:
    :return:
    """
    # user_video = VideoColor(video)
    # user_video.get_video_frames(progress=True)
    pass


def test_get_video_frames_gets_correct_params(video):
    user_video = VideoColor(video)

    with pytest.raises(ValueError):
        user_video.get_video_frames(color_format="invalid_random_format")

    with pytest.raises(TypeError):
        user_video.get_video_frames(frame_color_count="1")

    with pytest.raises(TypeError):
        user_video.get_video_frames(progress=24)


def test_ordered_colors_are_correct_count(video):
    """
    test sorted colors gets correct params and returns correct color count
    :param video:
    """
    user_video = VideoColor(video)
    all_colors = user_video.get_video_frames()
    with pytest.raises(Exception):
        col_share.sort_order(object_description=all_colors, key_count="5")
    with pytest.raises(Exception):
        col_share.sort_order(object_description=all_colors, ascending="random")

    dominant_colors = col_share.sort_order(object_description=all_colors, key_count=6)
    assert len(dominant_colors) == 6
    """
    below line might fail as colors are grabbed on the second instead of per frame
    hence two consecutive calls might grab diff frames on the same second
    """
    # assert list(dominant_colors.values()) == [68.83, 22.48, 22.22, 21.7, 19.11, 17.77]


def test_validation_of_rgb_is_correct(image):
    """
    test a valid rgb format can be identified
    """
    user_image = ColorDetect(image)
    assert user_image._validate_rgb((255, 0, 0))
    assert not user_image._validate_rgb((256, 0, 0))
    assert not user_image._validate_rgb((255, -2, 0))


def test_get_time_frame_color_returns_video_colors_at_given_time(video):
    """
    test whether you can get colors from video at specific time
    :param video
    """
    user_video = VideoColor(video)

    (image, colors_at_time) = user_video.get_time_frame_color(time=10000)
    assert isinstance(image, ColorDetect)
    assert isinstance(colors_at_time, dict)

    assert len(colors_at_time) == 5


def test_get_time_frame_gets_right_params(video):
    """
    test validity of params parsed to get time fram
    """
    user_video = VideoColor(video)

    with pytest.raises(ValueError):
        user_video.get_time_frame_color(color_format="invalid_random_format")

    with pytest.raises(TypeError):
        user_video.get_time_frame_color(color_count="45")

    # give a negative time to get color from
    with pytest.raises(ValueError):
        user_video.get_time_frame_color(time=-1000)

    # give a time period greater than the video
    with pytest.raises(ValueError):
        user_video.get_time_frame_color(time=100_000_000_000)



"""
.. _module_ColorDetect:
Module ColorDetect
==================
Defines ColorDetect class

For example:

>>> from colordetect import ColorDetect
>>> user_image = ColorDetect("<path_to_image>")
# where color_count is the target most dominant colors to be found. Default set to 5
>>> colors =  user_image.get_color_count(color_count=5)
>>> colors
# alternatively, save these RGB values to the image
>>> user_image.write_color_count()
>>> user_image.save_image("<storage_path>","<image_file_name>")
# Image processed and saved successfully
"""

import logging
import tempfile
from pathlib import Path
from urllib.request import urlopen

import cv2
import matplotlib.colors as mcolors
import numpy as np
import webcolors
from sklearn.cluster import KMeans

from . import col_share

LOGGER = logging.getLogger(__name__)


class ColorDetect:
    """
    Detect and recognize the number of colors in an image
    """

    def __init__(self, image, resize_h: int = None):
        """Create ColorDetect object by providing an image"""

        #  check type of data being passed
        if isinstance(image, np.ndarray):
            self.image = image
        elif isinstance(image, str):
            if col_share.is_url(image) is True:
                img_data = urlopen(image).read()
                dir_name = tempfile.mkdtemp()
                dir_path = Path(dir_name)
                # image saved in a temp dir as img.jpg to be read by colordetect
                dist_file = str(dir_path / "img.jpg")
                f = open(dist_file, "wb")
                f.write(img_data)
                f.close()
                self.image = cv2.imread(dist_file)
            else:
                self.image = cv2.imread(image)
        else:
            raise TypeError(
                "The image parameter accepts a numpy array , string file path or string file url argument only"
            )
        if resize_h is not None:
            h0, w0, _ = self.image.shape
            h1 = resize_h
            w1 = int(w0 * h1 / h0)
            self.image = cv2.resize(self.image, (w1, h1))
        self.image_original = self.image.copy()

        self.color_description = {}

    def get_segmented_image(
        self,
        lower_bound: tuple,
        upper_bound: tuple,
        erode_iterations: int = 3,
        dilate_iterations: int = 3,
        use_grab_cut: bool = True,
        gc_iterations: int = 3,
    ) -> tuple:
        """
        .. _get_segmented_image:
        get_segmented_image
        ---------------
        Get image masks from an image

        Parameters
        ----------
        lower_bound: tuple
            A lower color range from which to look from
        upper_bound: tuple
            The higher RGB color range from which to look from
        erode_iterations: int
            The number of times to perform erosion of the image
        dilate_iterations: int
            The number of times dilation is applied.
        use_grab_cut: bool
            A boolean indicating whether grabCut will be applied to the image. This is True by default.
        gc_iterations: int
            Number of iterations the algorithm should make before returning the result
        :return: output_image, gray, segmented, mask
        """

        if not self._validate_rgb(lower_bound):
            raise TypeError(
                f"lower_bound has to be a tuple of integers. Provided {type(lower_bound)} "
            )
        if not self._validate_rgb(upper_bound):
            raise TypeError(
                f"upper_bound has to be a tuple of integers. Provided {type(upper_bound)} "
            )

        if not isinstance(erode_iterations, int):
            raise TypeError(
                f"erode_iterations has to be an integer. Provided {type(erode_iterations)} "
            )

        if not isinstance(dilate_iterations, int):
            raise TypeError(
                f"dilate_iterations has to be an integer. Provided {type(dilate_iterations)} "
            )

        if not isinstance(gc_iterations, int):
            raise TypeError(
                f"gc_iterations has to be a an integer. Provided {type(gc_iterations)} "
            )
        if not isinstance(use_grab_cut, bool):
            raise TypeError(
                f"use_grab_cut has to be a boolean. Provided {type(use_grab_cut)} "
            )
        gray = cv2.cvtColor(self.image_original, cv2.COLOR_BGR2GRAY)
        output_image = cv2.cvtColor(gray, cv2.COLOR_GRAY2BGR)
        img2 = self.image_original.copy()
        img2 = cv2.GaussianBlur(img2, (11, 11), 0)
        img2 = cv2.cvtColor(img2, cv2.COLOR_BGR2HSV)

        mask = cv2.inRange(img2, lower_bound, upper_bound)
        mask = cv2.erode(mask, None, iterations=erode_iterations)
        mask = cv2.dilate(mask, None, iterations=dilate_iterations)

        if use_grab_cut:
            mask[mask == 0] = cv2.GC_BGD
            mask[mask > 0] = cv2.GC_PR_FGD

            fg_model = np.zeros((1, 65), dtype="float")
            bg_model = np.zeros((1, 65), dtype="float")

            mask, bg_model, fg_model = cv2.grabCut(
                self.image_original,
                mask,
                None,
                fg_model,
                bg_model,
                iterCount=gc_iterations,
                mode=cv2.GC_INIT_WITH_MASK,
            )
            mask = np.where((mask == cv2.GC_BGD) | (mask == cv2.GC_PR_BGD), 0, 1)
            mask = (mask * 255).astype("uint8")

        segmented = cv2.bitwise_and(self.image_original, self.image_original, mask=mask)

        for i in range(len(mask)):
            for j in range(len(mask[i])):
                if mask[i][j] != 0:
                    output_image[i][j] = self.image_original[i][j]

        return output_image, gray, segmented, mask

    def get_color_count(
        self, color_count: int = 5, color_format: str = "human_readable"
    ) -> dict:
        """
        .. _get_color_count:
        get_color_count
        ---------------
        Count the number of different colors


        Parameters
        ----------
        color_count: int
            The number of most dominant colors to be obtained from the image
        color_format:str
            The format to return  the color in.
            Options
                * hsv - (60°,100%,100%)
                * rgb - rgb(255, 255, 0) for yellow
                * hex - #FFFF00 for yellow
                * human_readable - yellow for yellow
        :return: color description
        """

        if not isinstance(color_count, int):
            raise TypeError(
                f"color_count has to be an integer. Provided {type(color_count)} "
            )

        # convert image from BGR to RGB for better accuracy
        rgb = cv2.cvtColor(self.image, cv2.COLOR_BGR2RGB)
        reshape = rgb.reshape((rgb.shape[0] * rgb.shape[1], 3))
        cluster = KMeans(n_clusters=color_count).fit(reshape)

        unique_colors = self._find_unique_colors(cluster, cluster.cluster_centers_)

        color_format_options = ["rgb", "hex", "hsv", "human_readable"]

        if color_format not in color_format_options:
            raise ValueError(f"Invalid color format: {color_format}")

        # round  up figures
        for percentage, v in unique_colors.items():
            rgb_value = list(np.around(v))
            if color_format != "rgb":
                color_value = self._format_color(v, color_format)
                self.color_description[color_value] = round(percentage, 2)
            else:
                self.color_description[str(rgb_value)] = round(percentage, 2)

        return self.color_description

    def _format_color(self, rgb_value, color_format: str):
        """
        Get the correct color format as specified
        :return:
        """
        if color_format == "hsv":
            # list(np.around(v))
            return str(mcolors.rgb_to_hsv(rgb_value).tolist())

        elif color_format == "hex":
            rgb_value = np.divide(rgb_value, 255)  # give a scale from 0-1
            return mcolors.to_hex(rgb_value)

        elif color_format == "human_readable":
            r0, g0, b0 = int(rgb_value[0]), int(rgb_value[1]), int(rgb_value[2])
            try:
                nearest = webcolors.rgb_to_name((r0, g0, b0))
            except (
                ValueError
            ):  # Calculate distances between rgb value and CSS3 rgb colours to determine the closest
                distances = {}
                for k, v in webcolors.CSS3_HEX_TO_NAMES.items():
                    r1, g1, b1 = webcolors.hex_to_rgb(k)
                    distances[((r0 - r1) ** 2 + (g0 - g1) ** 2 + (b0 - b1) ** 2)] = (
                        v  # Ignore sqrt as it has no significant effect
                    )
                nearest = distances[min(distances.keys())]
            return nearest

    def _find_unique_colors(self, cluster, centroids) -> dict:

        # Get the number of different clusters, create histogram, and normalize
        labels = np.arange(0, len(np.unique(cluster.labels_)) + 1)
        (hist, _) = np.histogram(cluster.labels_, bins=labels)
        hist = hist.astype("float")
        hist /= hist.sum()

        # iterate through each cluster's color and percentage
        colors = [(percent * 100, color) for (percent, color) in zip(hist, centroids)]
        colors.sort(key=lambda x: (x[0], id(x[1])))

        for percent, color in colors:
            color.astype("uint8").tolist()
        return dict(colors)

    def write_color_count(
        self,
        left_margin: int = 10,
        top_margin: int = 20,
        font: int = cv2.FONT_HERSHEY_SIMPLEX,
        font_color: tuple = (0, 0, 0),
        font_scale: float = 1.0,
        font_thickness: float = 1,
        line_type: int = 1,
        save: bool = False,
    ):
        """
        .. _write_color_count:
        write_color_count
        -----------------
        Write the number of colors found to the image

        Parameters
        ----------
        left_margin: int
            Text spacing from the left
        top_margin: int
            Text spacing from the top
        font: int
            Font to use in text. Look up acceptable values from python-opencv
        font_color:
            RGB tuple of text font color
        font_scale:
            Size of the text to be written
        font_thickness:
            Thickness of the text
        line_type: int = 1,
        """
        if not self.color_description:
            raise AttributeError(
                "No color description found on this object. Perform get_color_count() first."
            )

        if not self._validate_rgb(font_color):
            raise TypeError(
                f"font_color has to be a tuple of integers. Provided {font_color} "
            )

        for k, v in self.color_description.items():
            color_values = str(v) + "% :" + k
            (text_width, text_height), baseline = cv2.getTextSize(
                color_values, font, font_scale, font_thickness
            )
            # change to BGR color format from RGB tuple
            bgr_color_format = (font_color[2], font_color[1], font_color[0])
            self.write_text(
                text=color_values,
                left_margin=left_margin,
                top_margin=top_margin,
                font=font,
                font_color=bgr_color_format,
                font_scale=font_scale,
                font_thickness=font_thickness,
                line_type=line_type,
            )

            top_margin += text_height

        if save:
            self.save_image()

    def write_text(
        self,
        text: str = "",
        left_margin: int = 10,
        top_margin: int = 20,
        font: int = cv2.FONT_HERSHEY_SIMPLEX,
        font_color: tuple = (0, 0, 0),
        font_scale: float = 1.0,
        font_thickness: float = 1.0,
        line_type: int = 1,
        line_spacing: int = 0,
    ):
        """
        .. _write_text:
        write_text
        ----------
        Write text onto an image

         Parameters
        ----------
        text: str
            The text to be written onto the image
        line_spacing:int
            The spacing between lines
        left_margin: int
            Text spacing from the left
        top_margin: int
            Text spacing from the top
        font: int
            Font to use in text. Look up acceptable values from python-opencv
        font_color:
            RGB tuple of text font color
        font_scale:
            Size of the text to be written
        font_thickness: float = 1.0
            Thickness of the text
        line_type: int = 1,
            Space betweeen the lines
        :return:
        """
        if not isinstance(text, str):
            raise TypeError(
                f"text should be a string.Provided {text} of type {type(text)}"
            )

        if text == "":
            raise IOError("text should not be empty")

        if not self._validate_rgb(font_color):
            raise TypeError(
                f"font_color has to be a tuple of integers. Provided {font_color} "
            )

        # change to BGR color format from RGB tuple
        font_color = (font_color[2], font_color[1], font_color[0])

        cv2.putText(
            self.image,
            text,
            (left_margin, top_margin),
            font,
            font_scale,
            font_color,
            font_thickness,
            line_type,
            line_spacing,
        )

    def save_image(self, location: str = ".", file_name: str = "out.jpg"):
        """
        .. _save_image:
        save_image
        ----------------

        Save the resultant image file to the local directory

        Parameters
        ----------
        location: str
            The file location of the image
        file_name:str
            The name of the new image

        """

        image_folder = Path(location)

        if not image_folder.exists():
            raise NotADirectoryError("The storage folder does not exist.")

        if not isinstance(file_name, str):
            raise TypeError(f"file_name should be a string.Provided {type(file_name)}")

        image_to_save = image_folder / file_name

        # Save image
        cv2.imwrite(str(image_to_save), self.image)

        LOGGER.info("Image processed and saved successfully")

    def _validate_rgb(self, rgb_tuple: tuple) -> bool:
        """
        Validate whether a tuple passed is a valid RGB

        Parameters
        ----------
        rgb_tuple: tuple
            An RGB tuple color.
        :return:
        """
        tuple_made_of_three = isinstance(rgb_tuple, tuple) and len(rgb_tuple) == 3
        tuple_has_integers_only = (
            isinstance(rgb_tuple[0], int)
            and isinstance(rgb_tuple[1], int)
            and isinstance(rgb_tuple[2], int)
        )
        color_range = []

        for color in rgb_tuple:
            if color in range(0, 256):
                color_range.append(True)
            else:
                color_range.append(False)

        invalid_color_range = False in color_range

        return (
            tuple_made_of_three and tuple_has_integers_only and not invalid_color_range
        )



"""
.. _module_VideoColor:
Module VideoColor
==================
Defines VideoColor class

Usage:

>>> from colordetect import VideoColor
>>> user_video = VideoColor("<path_to_video>")
# where frame_color_count is the target most dominant colors to be found. Default set to 5
>>> colors =  user_video.get_video_frames(frame_color_count=7)
>>> colors
# alternatively shorten the dictionary to get a specific number of sorted colors from the whole lot
>>> from colordetect import col_share
>>> top_colors = col_share.sort_order(object_description=colors, key_count=8)
"""

import datetime
import sys

import cv2

from . import col_share
from .color_detect import ColorDetect


class VideoColor:
    """
    Detect and recognize the number of colors in a video
    """

    def __init__(self, video):
        # super().__init__(video)
        self.video_file = cv2.VideoCapture(video)
        self.color_description = {}

    def get_video_frames(
        self,
        frame_color_count: int = 5,
        color_format: str = "rgb",
        progress: bool = False,
    ) -> dict:
        """
        .. _get_video_frames:
        get_video_frames
        ----------------
        Get image frames and their colors from the video

        Parameters
        ----------
        frame_color_count: int
            The number of most dominant colors to be obtained from a single frame
        color_format:str
            The format to return the color in.
            Options
                * hsv - (60°,100%,100%)
                * rgb - rgb(255, 255, 0) for yellow
                * hex - #FFFF00 for yellow
          :return: color_description dictionary
        """
        if not isinstance(frame_color_count, int):
            raise TypeError(
                f"frame_color_count has to be an integer. Provided {type(frame_color_count)} "
            )

        color_format_options = ["rgb", "hex", "hsv"]

        if color_format not in color_format_options:
            raise ValueError(f"Invalid color format: {color_format}")

        if not isinstance(progress, bool):
            raise TypeError(f"Progress should be a boolean. Provided {type(progress)}")

        count = 0
        total_frame_count = self.video_file.get(cv2.CAP_PROP_FRAME_COUNT)
        while self.video_file.isOpened():
            # how often to extract colors from video frames. Defaults to every 1 second
            (success, image) = self._get_frame(time=count * 1000)
            if not success:
                break  # Video is complete
            image_object = ColorDetect(image)
            colors = image_object.get_color_count(
                color_count=frame_color_count, color_format=color_format
            )
            # merge dictionaries as they are created
            self.color_description = {**self.color_description, **colors}
            count += 1
            current_frame_num = self.video_file.get(cv2.CAP_PROP_POS_FRAMES)
            if progress:
                col_share.progress_bar(
                    position=current_frame_num, total_length=total_frame_count
                )
        if progress:
            col_share.progress_bar(
                position=total_frame_count, total_length=total_frame_count
            )  # Cater for video with extra millis at the end that don't sum upto a full sec, and are thus skipped

        self.video_file.release()
        print("\n")
        return self.color_description

    def _get_frame(self, time: int = 1000) -> tuple:
        """
         .. _get_frame_color:
        get_frame_color
        ----------------
        Get image frame at specific time in a video

        Parameters
        ----------
        time: int
            Time to get color from in parsed image

          :return: ()
        """
        #  read file every x time in milliseconds
        self.video_file.set(cv2.CAP_PROP_POS_MSEC, time)
        success, image = self.video_file.read()

        return (success, image)

    def get_time_frame_color(
        self, color_count: int = 5, color_format: str = "rgb", time: int = 1000
    ) -> tuple:
        """
         .. _get_time_frame_color:

        get_time_frame_color
        ----------------
        Get color from a specific time in the video

         Parameters
        ----------
        time: int
            Time to get color from in video in milliseconds
        color_count: int
            Number of colors to return at the given time frame
        color_format:str
            The format to return the color in.
            Options
                * hsv - (60°,100%,100%)
                * rgb - rgb(255, 255, 0) for yellow
                * hex - #FFFF00 for yellow
          :return: (image, color_description)
        """
        color_format_options = ["rgb", "hex", "hsv"]

        if color_format not in color_format_options:
            raise ValueError(f"Invalid color format: {color_format}")

        if not isinstance(color_count, int):
            raise TypeError(
                f"color_count to extract has to be an integer. Provided {type(color_count)} "
            )

        if time < 1:
            raise ValueError("Cannot give negative time to extract color from")

        if self._get_video_length() < time:
            raise ValueError(
                f"The time given is longer than the video parsed. Provided {time} while length of video: {self._get_video_length()}"
            )

        (success, image) = self._get_frame(time)
        if success:
            image_object = ColorDetect(image)
            colors = image_object.get_color_count(
                color_count=color_count, color_format=color_format
            )

            self.color_description = colors

        self.video_file.release()

        return (image_object, self.color_description)

    def _get_video_length(self) -> int:
        """
          .. _get_video_length:
        _get_video_length
        ----------------
        get the length of a video

        return: the length of a video
        """
        frames = self.video_file.get(cv2.CAP_PROP_FRAME_COUNT)
        fps = self.video_file.get(cv2.CAP_PROP_FPS)

        return round(frames / fps) * 1000
